<?php

/**
 * Mock file system, network, db functions for easier unit testing.
 */
class Mocks implements PMDependencies {
  private $_script = array();

  /**
   * The heart of our mini mock framework
   */
  public function returns($function, $value) {
    $this->_script[] = array(
      'function' => $function,
      'return_value' => $value,
      'action' => 'return',
      'acted' => FALSE,
    );
  }

  /**
   *
   */
  public function throws($function, $exception) {
    $this->_script[] = array(
      'function' => $function,
      'return_value' => $exception,
      'action' => 'throw',
      'acted' => FALSE,
    );
  }

  /**
   *
   */
  public function _process_request($function) {
    $line = array('action' => 'return', 'return_value' => NULL);

    // Find the next line of the script to play.
    foreach ($this->_script AS &$potential) {
      if ($potential['function'] == $function) {
        $line = $potential;

        // If the line has been played, search for an unplayed line.
        if (!$potential['acted']) {
          $potential['acted'] = TRUE;
          break;
        }
      }
    }

    // Either return the selected value or throw the exception.
    if ('throw' == $line['action']) {
      throw $line['return_value'];
    }

    return $line['return_value'];
  }

  /**
   * Functions being mocked.
   */

  public function file_directory_temp() {
    return $this->_process_request(__FUNCTION__);
  }

  public function file_exists($file) {
    return $this->_process_request(__FUNCTION__);
  }

  public function file_save_data($data, $file) {
    return $this->_process_request(__FUNCTION__);
  }

  public function file_scan_directory($directory, $pattern) {
    return $this->_process_request(__FUNCTION__);
  }

  public function http_request($url) {
    return $this->_process_request(__FUNCTION__);
  }

  public function rename($source, $dest) {
    return $this->_process_request(__FUNCTION__);
  }

  public function conf_path() {
    return $this->_process_request(__FUNCTION__);
  }
  
  public function create_project($builder, $project) {
    return $this->_process_request(__FUNCTION__);
  }

  public function get_queue() {
    return $this->_process_request(__FUNCTION__);
  }

  public function set_queue($queue = NULL) {
    return $this->_process_request(__FUNCTION__);
  }
}
