<?php

/**
 * @file
 * Object representation of any project hosted on Drupal.org
 */

class DOProject implements Project {
  private $fields;

  /**
   * Build a project out of a short_name.
   */
  function __construct($id) {
    $query = "SELECT title, short_name, links
              FROM {plugin_manager_do_repo}
              WHERE short_name = '%s'";

    // Save the results and return.
    $this->fields = db_fetch_array(db_query($query, $id));
  }

  function __sleep() {
    return array('fields');
  }
  
  /**
  * Get a list of all field and field metadata provided by this project.
  */
  public function get_fields() {
    return array(
      'id' => check_plain($this->fields['short_name']),
      'title' => check_plain($this->fields['title']),
      'links' => valid_url($this->fields['links'], TRUE) ? $this->fields['links'] : '',
    );
  }

  /**
   * Get the template function to use for rendering these results.
   */
  public function get_template($minimal) {
    return NULL;	  
  }
 
  /**
   * Get the release information for a specific release version 
   * 
   * @param $version
   *   The version to get release information for.
   * @return
   *   An array containing all available release information.
   */
  public function get_releases() {
    $release_list = array();
    $version = constant("DRUPAL_CORE_COMPATIBILITY");
    
    // Look up the data for every project requested.
    // @Todo: Check $short_name to make sure it is something safe.
    $file = drupal_http_request("http://updates.drupal.org/release-history/". $this->fields['short_name'] ."/$version");
    $xml = simplexml_load_string($file->data);

    // If it can't find the info for this project, skip to the next one.
    if ($xml == FALSE){
      throw new Exception(t('Downloading the release history failed for @project.', array('@project' => $this->fields['short_name'])));
    }

    // Find out the recommended and supported versions so that we can
    // later prioritize.
    $rec = (string)$xml->recommended_major;
    $sup = explode(",", $xml->supported_majors);

    // Get the information about every release
    foreach ($xml->releases->release AS $release) {
      if ((string)$release->status == 'published') {
        $ver = (string)$release->version;
        $release_list[$ver] = array(
          'version' => $ver,
          'release_link'  => (string)$release->release_link,
          'download_link' => (string)$release->download_link,
          'version_major' => (string)$release->version_major,
          'version_patch' => (string)$release->version_patch,
          'version_extra' => (string)$release->version_extra,
          'recommended' => ($rec == (string)$release->version_major),
          'supported' => in_array((string)$release->version_major, $sup),
        );
      }
    }

    // Order them and then return the results.
    uasort($release_list, array($this, '_sort_releases'));
    $release_list = array_reverse($release_list);
    return $release_list;
  }

  /**
   * Fetch all of the files from a given release.
   *
   * @return
   *   An array containing all of the base directories.
   */
  public function fetch_release($version) {
    $releases = $this->get_releases();
    $results = array(
      'id' => check_plain($this->fields['short_name']),
      'compression' => 'gzip',
      'custom' => FALSE,
      'download' => $releases[$version]['download_link'],
      'title' => $this->fields['title'],
    );

    return $results;
  }

  /**
   * Helper function to sort projects by priority.
   */
  private function _sort_releases($a, $b) {
    $hierarchy = array('recommended', 'supported', 'version_major', 'version_patch', 'version_extra');
    foreach($hierarchy AS $level) {
      if ($a[$level] != $b[$level]) {
        return $a[$level] - $b[$level];
      }  
    }
    return 0;
  }
}
