<?php
/**
 * Untars a file.
 *
 * @param $file
 *   The file to untar.
 * @return
 *   An array containing the locations of the extracted files.
 */
function plugin_manager_extract($file) {
  module_load_include('inc', 'plugin_manager', 'plugin_manager.tar');
  $depends = plugin_manager_dependencies();
  $dir = $depends->file_directory_temp();

  // Attempt to explode the file with Archive_Tar.
  $tar = new Archive_Tar($file);
  if (!$tar->extract($dir)) {
    throw new Exception('An error occurred while extracting files.');
  }

  // Find the top level file/folders extracted.
  $top_level = array();
  foreach ($tar->listcontent() AS $file) {
    $file_path = explode('/', $file['filename']);
    $file_root = $file_path[0];
    $top_level[$file_root] = $dir .'/'. $file_root;
  }
  return $top_level;
}

/**
 * Identify a folder as a module, theme or library.
 */
function plugin_manager_guess_type($folder) {
  $depends = plugin_manager_dependencies();
  $module = $depends->file_scan_directory($folder, '\.module$');
  $info = $depends->file_scan_directory($folder, '\.info$');

  if (!empty($module)) {
    return 'modules';
  }
  elseif (!empty($info)) {
    return 'themes';
  }
  else {
    return 'libraries';
  }
}

/**
 * Process the queue items.
 */
function plugin_manager_install_batch($instructions, &$context) {
  $depends = plugin_manager_dependencies();

  if (empty($context['sandbox'])) {
    $context['sandbox']['stages'] = $instructions['actions'];
  }

  try {
    $action = array_shift($context['sandbox']['stages']);
    $context['finished'] = 1 - (count($context['sandbox']['stages']) / count($instructions['actions']));
    
    switch ($action) {
      case PLUGIN_MANAGER_DOWNLOAD:
        // @Todo: Verify sites/conf_path()/$type/$project_name.
        $context['message'] = t("Downloading @plugin", array('@plugin' => $instructions['title']));
        $context['sandbox']['file'] = plugin_manager_get($instructions['download']);
        break;

      case PLUGIN_MANAGER_EXTRACT:
        $context['message'] = t("Extracting @plugin", array('@plugin' => $instructions['title']));
        $context['sandbox']['extract'] = plugin_manager_extract($context['sandbox']['file']);
        break;

      case PLUGIN_MANAGER_COPY:
        $context['message'] = t("Installing @plugin", array('@plugin' => $instructions['title']));
        foreach ($context['sandbox']['extract'] AS $source) {
          $dest = $depends->conf_path() .'/'. plugin_manager_guess_type($source) .'/'. basename($source);
          // @Todo: Check that files do not already exist.
          // @Todo: Check for rename success.
          $success = $depends->rename($source, $dest);
        }
        $context['result'][] = t("Installed @plugin", array('@plugin' => $instructions['title']));
        break;

      case PLUGIN_MANAGER_REMOVE:
        break;
    }
  }
  catch (Exception $e) {
    $context['result'][] = $e;
  }
}

/**
 * Landing page for finished queue processing.
 */
function plugin_manager_batch_finished($success, $results, $operations) {
  if ($success) {
    // Here we could do something meaningful with the results.
    foreach ($results AS $result) {
      drupal_set_message($result);
    }
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $message = 'An error occurred while processing ' . $error_operation[0] . ' with arguments :' . print_r($error_operation[0], TRUE);
    drupal_set_message($message);
  }
}
