Drupal.pluginManager = {};

/**
 * Setup the javascript for the plugin_manager.
 */
Drupal.behaviors.pluginManagerInit = function(context) {
  // Add a section to display messages.
  $('#plugin-manager-search-form').prepend('<div id="plugin-manager-messages"></div>');

  // Check for backends
  if (!Drupal.settings.plugin_manager.backends) {
    $('#plugin-manager-search-form input').attr('disabled', 'disabled');
    Drupal.pluginManager.showMessage(Drupal.t('Please enable a backend module before continuing.'), 'error');
    return false;
  }
	
  $('#edit-search').click(function () {
    Drupal.pluginManager.search();
    return false;
  });
  
  $('#edit-queued').click(function () {
    Drupal.pluginManager.search('queued');
    $('.fieldset-wrapper').after('<button id="clear_queue" class="removable action">' + Drupal.t('Clear Queue') + '</button>');
    $('#clear_queue').click(Drupal.pluginManager._clearQueue);
    return false;
  });
  
  Drupal.pluginManager.send('init');
};

/**
 * Post data back to the server and process results.
 */
Drupal.pluginManager.send = function(op, data, callback) {
  if (data == undefined) {
    data = {};
  }
  
  data.op = op;
  $.ajax({
    type: 'POST',
    url: Drupal.settings.basePath + '?q=admin/build/plugin_manager/ajax',
    data: data,
    // Always process results first THEN pass then along for anything else to process.
    success: function (response) {
      Drupal.pluginManager.processJSON(response);
      if (callback != undefined) {
        callback(response);
      }
    },
    dataType: 'json'
  });
};

/**
 * Process the JSON results of send().
 */
Drupal.pluginManager.processJSON = function(response) {
  // If messages are set, display them.
  if (response['message']) {
    for (var i in response['message']) {
      var message = response['message'][i];
      Drupal.pluginManager.showMessage(message['text'], message['severity'], message['timeout']);
    }
  }

  // If results need to be displayed, do so.
  if (response['results']) {
    $('.fieldset-wrapper').empty();
    $('.fieldset-wrapper').html(response['results']);

    // If new results have been added, add triggers to them.
    $('.plugin_manager_result_action a').click(Drupal.pluginManager.processClick);
  }
}

Drupal.pluginManager.processClick = function() {
  if ($(this)[0] == undefined) {
    return;
  }

  var tag = $(this)[0];
  var action = tag.innerHTML
  var project = tag.hash.substring(1);
  Drupal.pluginManager.send('queue', {'id' : project}, function(results) {
    $('#project_' + project).replaceWith(results['action']);
    $('#project_' + project).click(Drupal.pluginManager.processClick);
  });
}

/**
 * Display a message to the user.
 */
Drupal.pluginManager.showMessage = function(text, style, timeout) {
  if (undefined == Drupal.pluginManager.totalMessageCount) {
    Drupal.pluginManager.totalMessageCount = 0;
  }

  var messageId = 'plugin-manager-message-' + (Drupal.pluginManager.totalMessageCount++);
  $('#plugin-manager-messages').append('<div id="' + messageId + '" class="messages ' + style + '">' + text + '</div>');
  $('#' + messageId).hide().slideDown('slow');

  // If the message has a timeout, then remove it after a specified amount of time.
  if (undefined != timeout) {
    setTimeout(function() {
      $('#' + messageId).slideUp('slow', function() {
        $('#' + messageId).remove()
      });
    }, timeout * 1000);
  }
}

/**
 * 
 */
Drupal.pluginManager.search = function () {
  // If the query string is empty then don't bother searching.
  var query = $('#edit-input').val();
  if (query == '') {
    return;
  }
  
  $('.removable').remove();
  $('.fieldset-wrapper').empty();
  $('.fieldset-wrapper').append('<div id="status">Searching...</div>');
  
  // Search for the plugin and display the results.
  Drupal.pluginManager.send('search', {'query' : query});
};
